package sb.ldif;

import java.io.ByteArrayInputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FilenameFilter;

import olivamail.core.common.IOUtils;
import sb.recoder.Recoder;

public class EncDec {

	private String plainCp = "windows-1251";
	private String encCp = "UTF-8";
	boolean encode = true;
	Recoder recoder = null;
	String extPtrn = ".*\\.[Ll][Dd][Ii][Ff]";
	
	public void process(String source, String destination, boolean encode, String plainCp, String encCp, String ext) throws Exception {
		if (plainCp != null)
			this.plainCp = plainCp;
		if (encCp != null)
			this.encCp = encCp;
		this.encode = encode;
		if (ext != null)
			extPtrn = ".*\\." + ext;
		
		recoder = new Recoder();		
		File src = new File(source);
		File dest = new File(destination);

		if(!src.exists()) {
			System.out.println("Source does not exist");
			System.exit(1);			
		}
		
		boolean destIsDir = dest.exists() && dest.isDirectory();
		
		if(src.isFile() && !destIsDir) {
			processFile(src, dest);
		} else if(src.isFile() && destIsDir) {
			processFile(src, new File(destination, src.getName()));
		} else if(src.isDirectory() && destIsDir) {
			File[] files = src.listFiles(new FilenameFilter() {
				public boolean accept(File dir, String name) {
					return name.matches(extPtrn);
				}
			});
			if (files != null && files.length > 0) {
				System.out.println("LDIF files in the derectory: " + files.length);
				for (int i = 0; i < files.length; i++) {
					processFile(files[i], new File(destination, files[i].getName()));						
				}
			}			
		} else if(src.isDirectory() && !destIsDir) {
			System.out.println("Source is the directory and the destinatination should be a directory");
			System.exit(1);
		} else {
			System.out.println("No applicable source and destination");			
		}
	}
		
	private void processFile(File src, File dest) throws Exception {
		System.out.println("Target file: " + dest.getPath());
		String srcText = null;
		if (encode && !plainCp.equals(encCp)) {
			recoder.recode(src, null, plainCp, encCp, false, null, true);
			srcText = recoder.recodedString;
		} else {
			srcText = IOUtils.getStringFromIS(new FileInputStream(src), -1,
					plainCp);
		}
		LdifParser ldif = new LdifParser();
		String target = ldif.getEncDecLines(srcText, encode, encCp);
		if (target != null) {
			ByteArrayInputStream bais = null;
			if (encode || plainCp.equals(encCp)) {
				bais = new ByteArrayInputStream(target.getBytes(encCp));
			} else {
				recoder.recode(null, null, encCp, plainCp, false, target, true);
				target = recoder.recodedString;
				bais = new ByteArrayInputStream(target.getBytes(plainCp));
			}
			IOUtils.writeFile(bais, dest);
		} else
			IOUtils.copyFile(src, dest);
	}
}
