package sb.ldif;

import java.io.File;
import java.io.FilenameFilter;
import java.net.URL;
import java.util.Properties;

import javax.swing.JOptionPane;

public class LdifEncDec {

	private static final String STR_USAGE = "Usage: java -jar sb-ldif.jar -s <source dir or file> -d <dest dir or file> -m <mode: encode (default) | decode> -p <plain codepage> -e <encoded codepage> [-x <file ext (Default: ldif)>]";
	private static final String ARG_SOURCE = "source";
	private static final String ARG_DEST = "dest";
	private static final String ARG_MODE = "mode";
	private static final String ARG_PLAIN_CP = "plain_cp";
	private static final String ARG_ENC_CP = "enc_cp";
	private static final String ARG_EXT = "ext";
	private Properties propArgs = null;
	
	/**
	 * @param args
	 */
	public static void main(String[] args) {
		boolean lowVer = false;
		String javaVer = System.getProperty("java.version");
		if(javaVer != null) { 
			int idx0 = javaVer.indexOf('.');
			int idx1 = -1;
			String ver0 = javaVer;
			String ver1 = "-1";
			if (idx0 != -1) {
				ver0 = javaVer.substring(0, idx0);
				idx1 = javaVer.indexOf('.', idx0 + 1);
				if (idx1 != -1)
					ver1 = javaVer.substring(idx0 + 1, idx1);
			}
			int iver0 = -1;
			int iver1 = -1;
			lowVer = true;
			try {
				iver0 = Integer.parseInt(ver0);
				iver1 = Integer.parseInt(ver1);
			} catch (Exception exc) {
				lowVer = false;
				exc.printStackTrace();
			}
			if (lowVer)
				if (iver0 >= 1)
					if (iver1 >= 4)
						lowVer = false;								
		}				
		
		if (lowVer) {
			String verMismatch = Context.myName + ": Java version 1.4 or higher is required. But current is "
					+ System.getProperty("java.version");
			System.out.println(verMismatch);
			try {
				// AlertDialog.showErrorDialog(verMismatch);
				// AlertDialog cannot be used before the ClassPathHacker's job
				JOptionPane.showMessageDialog(null, verMismatch, "Error",
						JOptionPane.ERROR_MESSAGE);
			} catch (Exception e) {
				e.printStackTrace();
			}
			System.exit(1);
		}								
		
		//System.out.println(Context.myName + ": v1.1");
		
		if(args.length < 1) {
			System.out.println(STR_USAGE);
			System.exit(1);			
		}						
				
		URL url = Class.class.getResource(Context.mainClassPath);
		if(url == null) {
			System.out.println(Context.myName + ": Cannot get the URL for the class: " + 
					Context.mainClassPath);
			System.exit(1);
		}		
		
		String homeDir = url.getFile();
		if (homeDir.startsWith("file:"))
			homeDir = homeDir.substring(5);
		int idx = homeDir.indexOf('!');
		if (idx != -1)
			homeDir = homeDir.substring(0, idx);
		idx = homeDir.lastIndexOf('/');
		if (idx != -1)
			homeDir = homeDir.substring(0, idx);
		idx = homeDir.lastIndexOf('/');
		if (idx != -1) {
			if(homeDir.substring(idx + 1).toLowerCase().equals("lib"))
				homeDir = homeDir.substring(0, idx);
		}
		homeDir = homeDir.replaceAll("%20", " ");			
		
		Context.homeDir = homeDir + "/";
		System.out.println(Context.myName + ": Assuming " + Context.myName + " home dir is: " + homeDir);

		try {
			ClassPathHacker.addFile(Context.homeDir + "conf");
			
			File jmDir = new File(Context.homeDir + "lib/");
			if(jmDir.exists()) {
				File jars[] = jmDir.listFiles(new FilenameFilter() {
					public boolean accept(File dir, String name) {
						return (name.toLowerCase().indexOf(".jar") == name.length() - 4);
					}					
				});
				for(int i = 0; i < jars.length; i++) {
					ClassPathHacker.addFile(Context.homeDir + "lib/" + jars[i].getName());		
				}
			}						
//			ClassPathHacker.addFile(Context.homeDir + "lib/" + "activation.jar");
		} catch (Exception e) {
			e.printStackTrace();
		}			

		LdifEncDec ldifEnc = new LdifEncDec();
		ldifEnc.go(args);
	}

	public void go(String[] args) {
		parseArgs(args);
		try {
			boolean encode = true;
			if (propArgs.getProperty(ARG_MODE) != null)
				encode = ((String) propArgs.getProperty(ARG_MODE))
						.equalsIgnoreCase("encode");
			(new EncDec()).process(propArgs.getProperty(ARG_SOURCE), propArgs
					.getProperty(ARG_DEST), encode, propArgs
					.getProperty(ARG_PLAIN_CP), propArgs
					.getProperty(ARG_ENC_CP), propArgs.getProperty(ARG_EXT));
		} catch (Exception exc) {
			exc.printStackTrace();
			System.exit(1);
		}
	}
	
	private void parseArgs(String[] args) {
		propArgs = new Properties();
		for (int i = 0; i < args.length; i++) {
/*			
			if (i + 1 >= args.length) {
				System.out.println("args(limit)" + i + ":" + args[i]);
				System.out.println(STR_USAGE);
				System.exit(1);
			}
*/			
//			System.out.println("args" + i + ":" + args[i]);
			if (args[i].equalsIgnoreCase("-s")) {
				propArgs.put(ARG_SOURCE, args[++i]);
			} else if (args[i].equalsIgnoreCase("-d")) {
				propArgs.put(ARG_DEST, args[++i]);
			} else if (args[i].equalsIgnoreCase("-m")) {
				propArgs.put(ARG_MODE, args[++i]);
			} else if (args[i].equalsIgnoreCase("-p")) {
				propArgs.put(ARG_PLAIN_CP, args[++i]);
			} else if (args[i].equalsIgnoreCase("-e")) {
				propArgs.put(ARG_ENC_CP, args[++i]);
			} else if (args[i].equalsIgnoreCase("-x")) {
				propArgs.put(ARG_EXT, args[++i]);
			}
		}
		
		if (propArgs.get(ARG_SOURCE) == null || propArgs.get(ARG_DEST) == null) {
			System.out.println(STR_USAGE);
			System.exit(1);
		}
		
		if (propArgs.get(ARG_MODE) != null
				&& !(((String) propArgs.get(ARG_MODE)).equalsIgnoreCase("encode")
				|| ((String) propArgs.get(ARG_MODE)).equalsIgnoreCase("decode"))) {
			System.out.println(STR_USAGE);
			System.exit(1);
		}				
	}
		
}
