/*
 * @(#)LdifParser.java
 *
 * Copyright (C) 2009 Sergey Bredikhin
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package sb.ldif;

import java.util.Enumeration;
import java.util.StringTokenizer;
import java.util.Vector;
import olivamail.core.common.ComUtils;
import olivamail.core.mail.Coder;

/**
 * 
 * @author Sergey Bredikhin
 * 
 */
public class LdifParser {
	private StringTokenizer st = null;
	private String nextLine = null;
	private boolean encode = true;
	private Vector singleValueLines = new Vector();
	private Vector outLines = new Vector();
	private boolean changed = false;
	private String outcp = "UTF=8";
	private static final String NEW_LINE = "\r\n";
	
	public LdifParser() {
	}
	
	public String getEncDecLines(String source, boolean encode,
			String encCp) throws Exception {
		changed = false;
		this.outcp = ComUtils.nvl(encCp, this.outcp);
		st = new StringTokenizer(source, NEW_LINE);
		this.encode = encode;

		while (moreToEncodeDecode())
			;

		if (changed) {
			StringBuffer sb = new StringBuffer();
//			boolean first = true;
			for (Enumeration en = outLines.elements(); en.hasMoreElements();) {
/*				
				if(first)
					first = false;
				else
					sb.append(NEW_LINE);
*/					
				sb.append((String) en.nextElement()).append(NEW_LINE);
			}
			return sb.toString();
		}
		return null;
	}

	private boolean moreToEncodeDecode() throws Exception {
		String line = null;
		if (nextLine == null) {
			if (st.hasMoreTokens())
				line = st.nextToken();
		} else {
			line = nextLine;
			nextLine = null;
		}
		if (line == null)
			return false;

		if(line.trim().equals("-")) {
			outLines.add("-");
			return true;
		}

		if(line.trim().equals("")) {
			//outLines.add(NEW_LINE);
			return true;
		}		
		
		singleValueLines.clear();
		singleValueLines.add(line);
		
		while (st.hasMoreTokens()) {
			nextLine = st.nextToken();
			if (nextLine.indexOf(": ") == -1 && nextLine.indexOf(":: ") == -1
					&& !nextLine.trim().equals("-")
					&& !nextLine.trim().equals("")) {
				if (nextLine.charAt(0) == ' ' || nextLine.charAt(0) == '\t') {
					singleValueLines.add(nextLine);					
					line += nextLine.substring(1);
					nextLine = null;
				}
			} else
				break;
		}

		boolean localChanged = false;
		//System.out.println("line: " + line);
		String ptrn = ":: ";
		int idx = line.indexOf(ptrn);
		if (idx != -1) {
			if (!encode) {
				String attrName = line.substring(0, idx);
				String attrValue = line.substring(idx + ptrn.length());
				String decValue = (new Coder(outcp)).decode("=?" + outcp
						+ "?B?" + attrValue + "?=", outcp);
				localChanged = true;
				changed = true;
				line = attrName + ": " + decValue;
			}
		} else {
			if (encode) {
				ptrn = ": ";
				idx = line.indexOf(ptrn);
				String attrName = line.substring(0, idx);
				String attrValue = line.substring(idx + ptrn.length());
				String encValue = encodeBASE64ifNeeded(attrValue, outcp, false);
				if (attrValue != encValue) {
					localChanged = true;
					changed = true;
					line = attrName + ":: " + encValue;
				}
			}
		}								
		
		if(localChanged)
			outLines.add(line);
		else
			outLines.addAll(singleValueLines);
		
		return true;
	}
	
	public static String encodeBASE64ifNeeded(String source, String codePage,
			boolean forceUnicode) throws Exception {
		byte asciiBytes[] = source.getBytes("us-ascii");
		String asciiSource = new String(asciiBytes, "us-ascii");
		String result = null;
		if (source.equals(asciiSource))
			result = source;
		else {
			if (forceUnicode)
				result = Base64.encode(source, "UTF-8").trim();
			else
				result = Base64.encode(source, codePage).trim();
		}
		return result;
	}
		  
}